#extension GL_OES_EGL_image_external : require
precision mediump float;
varying highp vec2 vTextureCoord;
uniform lowp samplerExternalOES sTexture;
uniform highp float aspectRatio;

const vec2 CENTER = vec2(0.5, 0.5);
const float VIGNETTE_RADIUS = 0.75; //RADIUS of our vignette, where 0.5 results in a circle fitting the screen
const float VIGNETTE_SOFTNESS = 0.2; //softness of our vignette, between 0.0 and 1.0
const vec3 W = vec3(0.2125, 0.7154, 0.0721);

vec3 BrightnessContrastSaturation(vec3 color, float brt, float con, float sat)
{
	vec3 black = vec3(0., 0., 0.);
	vec3 middle = vec3(0.5, 0.5, 0.5);
	float luminance = dot(color, W);
	vec3 gray = vec3(luminance, luminance, luminance);

	vec3 brtColor = mix(black, color, brt);
	vec3 conColor = mix(middle, brtColor, con);
	vec3 satColor = mix(gray, conColor, sat);
	return satColor;
}

vec3 overlayBlender(vec3 Color, vec3 filt) {
	vec3 filter_result;
	float luminance = dot(filt, W);

	if(luminance < 0.5)
		filter_result = 2. * filt * Color;
	else
		filter_result = 1. - (1. - (2. *(filt - 0.5)))*(1. - Color);

	return filter_result;
}


void main() {
    //get the pixel
    vec2 st = vTextureCoord.st;
    vec3 rgb = texture2D(sTexture, st).rgb;

    //adjust the brightness/contrast/saturation
    float T_bright = 1.3;
    float T_contrast = .2;
    float T_saturation = 0.;
    vec3 bcs_result = BrightnessContrastSaturation(rgb, T_bright, T_contrast, T_saturation);

    //more red, less blue
    vec3 rb_result = vec3(bcs_result.r*1.25, bcs_result.g, bcs_result.b*0.8);

    //add filter (overlay blending)
    vec3 after_filter = mix(rb_result, overlayBlender(rb_result, rgb), 0.8);

    vec4 texColor = vec4(after_filter, 1.);
    highp vec2 adjustedCoord = vec2(vTextureCoord.x, (vTextureCoord.y * aspectRatio + 0.5 - 0.5 * aspectRatio));
    float dist = distance(adjustedCoord, CENTER);
    float vignette = smoothstep(VIGNETTE_RADIUS, VIGNETTE_SOFTNESS, dist * 2.);
    texColor.rgb = mix(texColor.rgb, texColor.rgb * vignette, 0.4);

    gl_FragColor = texColor;
}
